package sun.rmi.transport;

import java.util.Arrays;
import java.io.ObjectInput;
import java.io.IOException;
import java.rmi.Remote;
import java.io.ObjectOutput;
import java.rmi.RemoteException;
import sun.rmi.transport.tcp.TCPEndpoint;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.RMIClientSocketFactory;
import java.rmi.server.ObjID;

public class LiveRef implements Cloneable
{
    private final Endpoint ep;
    private final ObjID id;
    private transient Channel ch;
    private final boolean isLocal;
    
    public LiveRef(ObjID id, Endpoint ep, boolean isLocal) {
        this.ep = ep;
        this.id = id;
        this.isLocal = isLocal;
    }
    
    public LiveRef(int n) {
        this(new ObjID(), n);
    }
    
    public LiveRef(int n, RMIClientSocketFactory rmiClientSocketFactory, RMIServerSocketFactory rmiServerSocketFactory) {
        this(new ObjID(), n, rmiClientSocketFactory, rmiServerSocketFactory);
    }
    
    public LiveRef(ObjID objID, int port) {
        this(objID, TCPEndpoint.getLocalEndpoint(port), true);
    }
    
    public LiveRef(ObjID objID, int port, RMIClientSocketFactory csf, RMIServerSocketFactory ssf) {
        this(objID, TCPEndpoint.getLocalEndpoint(port, csf, ssf), true);
    }
    
    public Object clone() {
        try {
            return (Object)super.clone();
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause.toString(), cause);
        }
    }
    
    public int getPort() {
        return ((TCPEndpoint)this.ep).getPort();
    }
    
    public RMIClientSocketFactory getClientSocketFactory() {
        return ((TCPEndpoint)this.ep).getClientSocketFactory();
    }
    
    public RMIServerSocketFactory getServerSocketFactory() {
        return ((TCPEndpoint)this.ep).getServerSocketFactory();
    }
    
    public void exportObject(Target target) throws RemoteException {
        this.ep.exportObject(target);
    }
    
    public Channel getChannel() throws RemoteException {
        if (this.ch == null) {
            this.ch = this.ep.getChannel();
        }
        return this.ch;
    }
    
    public ObjID getObjID() {
        return this.id;
    }
    
    Endpoint getEndpoint() {
        return this.ep;
    }
    
    @Override
    public String toString() {
        String str;
        if (this.isLocal) {
            str = "local";
        }
        else {
            str = "remote";
        }
        return "[endpoint:" + this.ep + "(" + str + "),objID:" + this.id + "]";
    }
    
    @Override
    public int hashCode() {
        return this.id.hashCode();
    }
    
    @Override
    public boolean equals(Object o) {
        if (o != null && o instanceof LiveRef) {
            LiveRef liveRef = (LiveRef)o;
            return this.ep.equals(liveRef.ep) && this.id.equals(liveRef.id) && this.isLocal == liveRef.isLocal;
        }
        return false;
    }
    
    public boolean remoteEquals(Object o) {
        if (o != null && o instanceof LiveRef) {
            LiveRef liveRef = (LiveRef)o;
            TCPEndpoint tcpEndpoint = (TCPEndpoint)this.ep;
            TCPEndpoint tcpEndpoint2 = (TCPEndpoint)liveRef.ep;
            RMIClientSocketFactory clientSocketFactory = tcpEndpoint.getClientSocketFactory();
            RMIClientSocketFactory clientSocketFactory2 = tcpEndpoint2.getClientSocketFactory();
            return tcpEndpoint.getPort() == tcpEndpoint2.getPort() && tcpEndpoint.getHost().equals(tcpEndpoint2.getHost()) && !(clientSocketFactory == null ^ clientSocketFactory2 == null) && (clientSocketFactory == null || (clientSocketFactory.getClass() == clientSocketFactory2.getClass() && clientSocketFactory.equals(clientSocketFactory2))) && this.id.equals(liveRef.id);
        }
        return false;
    }
    
    public void write(ObjectOutput out, boolean b) throws IOException {
        boolean resultStream = false;
        if ((out instanceof ConnectionOutputStream)) {
            ConnectionOutputStream connectionOutputStream = (ConnectionOutputStream)out;
            resultStream = connectionOutputStream.isResultStream();
            if (this.isLocal) {
                Target target = ObjectTable.getTarget(new ObjectEndpoint(this.id, this.ep.getInboundTransport()));
                if (target != null) {
                    Remote impl = target.getImpl();
                    if (impl != null) {
                        connectionOutputStream.saveObject(impl);
                    }
                }
            }
            else {
                connectionOutputStream.saveObject(this);
            }
        }
        if (b) {
            ((TCPEndpoint)this.ep).write(out);
        }
        else {
            ((TCPEndpoint)this.ep).writeHostPortFormat(out);
        }
        this.id.write(out);
        out.writeBoolean((boolean)resultStream);
    }
    
    public static LiveRef read(ObjectInput in, boolean b) throws IOException, ClassNotFoundException {
        TCPEndpoint tcpEndpoint;
        if (b) {
            tcpEndpoint = TCPEndpoint.read(in);
        }
        else {
            tcpEndpoint = TCPEndpoint.readHostPortFormat(in);
        }
        ObjID read = ObjID.read(in);
        int boolean1 = in.readBoolean();
        LiveRef o = new LiveRef(read, tcpEndpoint, false);
        if ((in instanceof ConnectionInputStream)) {
            ConnectionInputStream connectionInputStream = (ConnectionInputStream)in;
            connectionInputStream.saveRef(o);
            if (!(boolean1 == 0)) {
                connectionInputStream.setAckNeeded();
            }
            return o;
        }
        else {
            DGCClient.registerRefs(tcpEndpoint, Arrays.asList(new LiveRef[] { o }));
            return o;
        }
    }
}
